AsCopyTarget

Struct AsCopyTarget 

pub struct AsCopyTarget<'lt, E> { /* private fields */ }
Borrows from a mutable data source to fill it with data from mages.

The type parameter is the layout engine which defines the byte spans of data to be copied and in doing so controls the overhead of the operation. Note that the type must implement a sealed trait for all the main algorithms. The respective constructors on DataRef, DataMut, DataCells choose this parameter.

Implementations

Source

impl<E: LayoutEngine> AsCopyTarget<'_, E>

pub fn read_from_ref(&mut self, buffer: ImageRef<'_, Bytes>)

Read out data from a borrowed buffer.

This reads data up to our layout. It does not interpret the data with the layout of the argument buffer.

pub fn read_from_cell_ref(&mut self, buffer: CellImageRef<'_, Bytes>)

Read out data from a borrowed buffer.

This reads data up to our layout. It does not interpret the data with the layout of the argument buffer.

pub fn read_from_atomic_ref(&mut self, buffer: CellImageRef<'_, Bytes>)

Read out data from a borrowed buffer.

This reads data up to our layout. It does not interpret the data with the layout of the argument buffer.