pub struct DataCells<'lt, Layout = Bytes> { /* private fields */ }Expand description
A cell buffer with layout, not aligned to any particular boundary.
Implementations
Sourceimpl<'lt, L> DataCells<'lt, L>
impl<'lt, L> DataCells<'lt, L>
Sourcepub fn with_layout_at(
data: &'lt [Cell<u8>],
layout: L,
start: usize,
) -> Option<Self>where
L: Layout,
pub fn with_layout_at(
data: &'lt [Cell<u8>],
layout: L,
start: usize,
) -> Option<Self>where
L: Layout,
Verifies the data against the layout before construction.
This wraps an shared buffer which has image data of the indicated layout from its start
byte onwards.
Sourcepub fn as_source(&self) -> AsCopySource<'_, RangeEngine<L>>
pub fn as_source(&self) -> AsCopySource<'_, RangeEngine<L>>
An adapter reading from the data as one contiguous chunk.
See RangeEngine for more explanations.
Sourcepub fn as_target(&mut self) -> AsCopyTarget<'_, RangeEngine<L>>
pub fn as_target(&mut self) -> AsCopyTarget<'_, RangeEngine<L>>
An adapter writing to this buffer in one contiguous chunk.
See RangeEngine for more explanations.