DataRef

Struct DataRef 

pub struct DataRef<'lt, Layout = Bytes> { /* private fields */ }
A buffer with layout, not aligned to any particular boundary.

Implementations

Source

impl<'lt> DataRef<'lt, Bytes>

pub fn new(data: &'lt [u8]) -> Self

Treat a whole input buffer as image bytes.

Source

impl<'lt, L> DataRef<'lt, L>

pub fn with_layout_at(data: &'lt [u8], layout: L, start: usize) -> Option<Self>
where L: Layout,

Construct from an explicit layout.

This wraps an underlying buffer which has image data of the indicated layout from its start byte onwards.

pub fn as_source(&self) -> AsCopySource<'_, RangeEngine<L>>
where L: Clone + Layout,

An adapter reading from the data as one contiguous chunk.

See RangeEngine for more explanations.