Bytes

Struct Bytes 

pub struct Bytes(pub usize);
A byte layout that only describes the user bytes.

This is a minimal implementation of the basic Layout trait. It does not provide any additional semantics for the buffer bytes described by it. All other layouts may be converted into this layout.

Tuple Fields

0: usize

Implementations

Source

impl Bytes

pub fn from_layout(layout: impl Layout) -> Self

Forget all layout semantics except the number of bytes used.