pub struct MismatchedPixelError { /* private fields */ }Expand description
An error indicating that mending failed due to mismatching pixel attributes.
This struct is used when a layout with dynamic pixel information should be mended into another layout with static information or a more restrictive combination of layouts. One example is the conversion of a dynamic matrix into a statically typed layout.