PlaneMatrices

Struct PlaneMatrices 

pub struct PlaneMatrices<T, const N: usize> { /* private fields */ }
An array of byte matrices.

This type is optimized for the concrete layout type of matrix planes.

Examples

use image_texel::image::Image;
use image_texel::layout::{PlaneMatrices, Matrix};
use image_texel::texels::U8;

// Imagine a JPEG with progressive DCT coefficient planes.
let rough = Matrix::from_width_height(U8, 8, 8).unwrap();
let dense = Matrix::from_width_height(U8, 64, 64).unwrap();

// The assembled layout can be used to access the disjoint planes.
let matrices = PlaneMatrices::new(U8, [rough, dense]);
let rough = matrices.plane_ref(0).unwrap();
let dense = matrices.plane_ref(1).unwrap();

let buffer = Image::new(&matrices);
let rough_coeffs = &buffer.as_buf()[rough.texel_range()];
assert_eq!(rough_coeffs.len(), 8 * 8);

let dense_coeffs = &buffer.as_buf()[dense.texel_range()];
assert_eq!(dense_coeffs.len(), 64 * 64);

// The coefficient planes are disjoint and well-ordered.
assert!(rough_coeffs.as_ptr_range().end <= dense_coeffs.as_ptr());

Implementations

Source

impl<T, const N: usize> PlaneMatrices<T, N>

pub fn new(texel: Texel<T>, inner: [Matrix<T>; N]) -> Self

Construct from separate matrices.

Relocates each consecutive matrix such that they do not overlap.

Panics

This method panics if the overall layout length would exceed isize::MAX.

pub fn from_repeated(matrix: Matrix<T>) -> Self

pub fn plane_ref(&self, idx: usize) -> Option<&Relocated<Matrix<T>>>

Return a reference to one relocated matrix layout.