pub struct StrideSpec {
pub width: usize,
pub height: usize,
pub element: TexelLayout,
pub width_stride: usize,
pub height_stride: usize,
pub offset: usize,
}Expand description
A simple layout describing some pixels as a byte matrix.
Fields
width: usizeThe number of pixels in width direction.
height: usizeThe number of pixels in height direction.
element: TexelLayoutThe number of bytes of a single pixel.
If this differs from both width_stride and height_stride the any copy must loop over
individual pixels. Otherwise, whole rows or columns of contiguous data may be inspected.
width_stride: usizeThe number of bytes to go one pixel along the width.
height_stride: usizeThe number of bytes to go one pixel along the height.
offset: usizeOffset of this matrix from the start.