TexelLayout

Struct TexelLayout 

pub struct TexelLayout { /* private fields */ }
Describes the byte layout of an texture element, untyped.

This is not so different from Texel and Layout but is a combination of both. It has the same invariants on alignment as the former which being untyped like the latter. The alignment of an element must be at most that of MaxAligned and the size must be a multiple of its alignment.

This type is a lower semi lattice. That is, given two elements the type formed by taking the minimum of size and alignment individually will always form another valid element. This operation is implemented in the Self::infimum method.

Implementations

Source

impl TexelLayout

pub const MAX_SIZE: Self

An element with maximum size and no alignment requirements.

This constructor is mainly useful for the purpose of using it as a modifier. When used with Self::infimum it will only shrink the alignment and keep the size unchanged.

pub fn from_pixel<P: AsTexel>() -> Self

Construct an element from a self-evident pixel.

pub fn with_layout(layout: Layout) -> Option<Self>

Create an element for a fictional type with specific layout.

It’s up to the caller to define or use an actual type with that same layout later. This skips the check that such a type must not contain any padding and only performs the layout related checks.

pub fn layout(self) -> Layout

Convert this into a type layout.

This can never fail as TexelLayout refines the standard library layout type.

pub fn packed(self, align: usize) -> TexelLayout

Reduce the alignment of the element.

This will perform the same modification as repr(packed) on the element’s type.

Panics

This method panics if align is not a valid alignment.

pub fn infimum(self, other: Self) -> TexelLayout

Create an element having the smaller of both sizes and alignments.

pub const fn size(self) -> usize

Get the size of the element.

pub const fn align(self) -> usize

Get the minimum required alignment of the element.

pub const fn superset_of(&self, other: TexelLayout) -> bool