pub struct MaxCell(/* private fields */);Expand description
A cell of a byte array equivalent to MaxAligned.
Implementations
Sourceimpl MaxCell
impl MaxCell
Sourcepub fn new(contents: MaxAligned) -> Self
pub fn new(contents: MaxAligned) -> Self
Create a vector from values initialized synchronously.
Sourcepub fn set(&self, newval: &Self)
pub fn set(&self, newval: &Self)
Overwrite the contents with new information from another cell.
Sourcepub fn get(&self) -> MaxAligned
pub fn get(&self) -> MaxAligned
Read the current contents from this cell into an owned value.
Sourcepub fn into_inner(self) -> MaxAligned
pub fn into_inner(self) -> MaxAligned
Unwrap an owned value.