pub struct AsCopyTarget<'lt, E> { /* private fields */ }Expand description
Borrows from a mutable data source to fill it with data from mages.
The type parameter is the layout engine which defines the byte spans of data to be copied and
in doing so controls the overhead of the operation. Note that the type must implement a sealed
trait for all the main algorithms. The respective constructors on DataRef, DataMut,
DataCells choose this parameter.
Implementations
Sourceimpl<E: LayoutEngine> AsCopyTarget<'_, E>
impl<E: LayoutEngine> AsCopyTarget<'_, E>
Sourcepub fn read_from_ref(&mut self, buffer: ImageRef<'_, Bytes>)
pub fn read_from_ref(&mut self, buffer: ImageRef<'_, Bytes>)
Read out data from a borrowed buffer.
This reads data up to our layout. It does not interpret the data with the layout of the argument buffer.
Sourcepub fn read_from_cell_ref(&mut self, buffer: CellImageRef<'_, Bytes>)
pub fn read_from_cell_ref(&mut self, buffer: CellImageRef<'_, Bytes>)
Read out data from a borrowed buffer.
This reads data up to our layout. It does not interpret the data with the layout of the argument buffer.
Sourcepub fn read_from_atomic_ref(&mut self, buffer: CellImageRef<'_, Bytes>)
pub fn read_from_atomic_ref(&mut self, buffer: CellImageRef<'_, Bytes>)
Read out data from a borrowed buffer.
This reads data up to our layout. It does not interpret the data with the layout of the argument buffer.