pub struct DataRef<'lt, Layout = Bytes> { /* private fields */ }Expand description
A buffer with layout, not aligned to any particular boundary.
Implementations
Sourceimpl<'lt, L> DataRef<'lt, L>
impl<'lt, L> DataRef<'lt, L>
Sourcepub fn with_layout_at(data: &'lt [u8], layout: L, start: usize) -> Option<Self>where
L: Layout,
pub fn with_layout_at(data: &'lt [u8], layout: L, start: usize) -> Option<Self>where
L: Layout,
Construct from an explicit layout.
This wraps an underlying buffer which has image data of the indicated layout from its
start byte onwards.
Sourcepub fn as_source(&self) -> AsCopySource<'_, RangeEngine<L>>
pub fn as_source(&self) -> AsCopySource<'_, RangeEngine<L>>
An adapter reading from the data as one contiguous chunk.
See RangeEngine for more explanations.