RangeEngine

Struct RangeEngine 

pub struct RangeEngine<L> { /* private fields */ }
Copies all bytes within the bounds of a layout.

This strategy will do a single copy, of the appropriate type for the targets buffer type, to transfer all the raw byte data of the image. This is optimal if the layout is supposed to be the first or only plane and does not contain any other internal padding buffers either.