pub struct ImageRef<'buf, Layout = &'buf Bytes> { /* private fields */ }
A read-only view of an image.

Note that this requires its underlying buffer to be highly aligned! For that reason it is not possible to take a reference at an arbitrary number of bytes.

Implementations

Get a reference to those bytes used by the layout.

Get a view of this image.

Check if a call to ImageRef::with_layout would succeed.

Change this view to a different layout.

This returns Some if the layout fits the underlying data, and None otherwise. Use ImageRef::fits to check this property in a separate call. Note that the new layout need not be related to the old layout in any other way.

Decay into a image with less specific layout.

See Image::decay.

Copy all bytes to a newly allocated image.

Get a slice of the individual samples in the layout.

View this buffer as a slice of pixels.

This reinterprets the bytes of the buffer. It can be used to view the buffer as any kind of pixel, regardless of its association with the layout. Use it with care.

An alternative way to get a slice of texels when a layout has an inherent texel type is Self::as_slice.

Turn into a slice of the individual samples in the layout.

This preserves the lifetime with which the layout is borrowed from the underlying image, and the ImageMut need not stay alive.

Retrieve a single texel from a raster image.