pub struct StridedBufferRef<'data> { /* private fields */ }
A reference to byte of a strided matrix.

Implementations

Construct a reference to a strided image buffer.

View bytes under a certain strided layout.

Unlike an image, the data need only be aligned to the element mentioned in the layout and not to the maximum alignment.

Shrink the element’s size or alignment.

Borrow this as a reference to a strided byte matrix.