pub struct Bytes(pub usize);Expand description
A byte layout that only describes the user bytes.
This is a minimal implementation of the basic Layout trait. It does not provide any
additional semantics for the buffer bytes described by it. All other layouts may be converted
into this layout.
Tuple Fields
0: usizeImplementations