pub struct Matrix<P> { /* private fields */ }Expand description
A matrix of packed texels (channel groups).
The underlying buffer may have more data allocated than this region and cause the overhead to be reused when resizing the image. All ways to construct this already check that all pixels within the resulting image can be addressed via an index.
Implementations
Sourceimpl<P> Matrix<P>
impl<P> Matrix<P>
pub fn from_width_height( pixel: Texel<P>, width: usize, height: usize, ) -> Option<Self>
pub fn width_and_height(width: usize, height: usize) -> Option<Self>where
P: AsTexel,
pub const fn empty(pixel: Texel<P>) -> Self
pub fn into_matrix_bytes(self) -> MatrixBytes
pub fn width(self) -> usize
pub fn height(self) -> usize
pub fn pixel(self) -> Texel<P>
Sourcepub fn transmute<Q: AsTexel>(self) -> Matrix<Q>
pub fn transmute<Q: AsTexel>(self) -> Matrix<Q>
Reinterpret to another, same size pixel type.
See transmute_to for details.
Sourcepub fn transmute_to<Q>(self, pixel: Texel<Q>) -> Matrix<Q>
pub fn transmute_to<Q>(self, pixel: Texel<Q>) -> Matrix<Q>
Reinterpret to another, same size pixel type.
Panics
Like std::mem::transmute, the size of the two types need to be equal. This ensures that
all indices are valid in both directions.
Sourceimpl<P> Matrix<P>
impl<P> Matrix<P>
pub fn with_matrix(pixel: Texel<P>, matrix: MatrixBytes) -> Option<Self>