pub struct Relocated<T> {
pub offset: AlignedOffset,
pub inner: T,
}Expand description
Moves a base layout to an aligned offset location.
This effectively allows turning one layout into a plane of another.
Fields
offset: AlignedOffsetinner: TImplementations
Sourceimpl<T: Layout> Relocated<T>
impl<T: Layout> Relocated<T>
pub fn new(inner: T) -> Self
Sourcepub fn next_aligned_offset(&self) -> Option<AlignedOffset>
pub fn next_aligned_offset(&self) -> Option<AlignedOffset>
Get the next aligned offset that comes after this relocated layout.
Sourcepub fn texel_range(&self) -> TexelRange<T::Sample>where
T: SliceLayout,
pub fn texel_range(&self) -> TexelRange<T::Sample>where
T: SliceLayout,
Get an index addressing all samples covered by the range of this relocated layout.