pub struct StrideSpec {
    pub width: usize,
    pub height: usize,
    pub element: TexelLayout,
    pub width_stride: usize,
    pub height_stride: usize,
    pub offset: usize,
}
A simple layout describing some pixels as a byte matrix.

Fields

width: usize

The number of pixels in width direction.

height: usize

The number of pixels in height direction.

element: TexelLayout

The number of bytes of a single pixel.

If this differs from both width_stride and height_stride the any copy must loop over individual pixels. Otherwise, whole rows or columns of contiguous data may be inspected.

width_stride: usize

The number of bytes to go one pixel along the width.

height_stride: usize

The number of bytes to go one pixel along the height.

offset: usize

Offset of this matrix from the start.