Struct image_texel::layout::TexelLayout
source · [−]pub struct TexelLayout { /* private fields */ }Expand description
Describes the byte layout of an texture element, untyped.
This is not so different from Texel and Layout but is a combination of both. It has the
same invariants on alignment as the former which being untyped like the latter. The alignment
of an element must be at most that of MaxAligned and the size must be a multiple of its
alignment.
This type is a lower semi lattice. That is, given two elements the type formed by taking the
minimum of size and alignment individually will always form another valid element. This
operation is implemented in the Self::infimum method.
Implementations
sourceimpl TexelLayout
impl TexelLayout
sourcepub fn from_pixel<P: AsTexel>() -> Self
pub fn from_pixel<P: AsTexel>() -> Self
Construct an element from a self-evident pixel.
sourcepub const MAX_SIZE: Self = {
TexelLayout {
size: isize::MAX as usize,
align: 1,
}
}
pub const MAX_SIZE: Self = { TexelLayout { size: isize::MAX as usize, align: 1, } }
An element with maximum size and no alignment requirements.
This constructor is mainly useful for the purpose of using it as a modifier. When used with
Self::infimum it will only shrink the alignment and keep the size unchanged.
sourcepub fn with_layout(layout: Layout) -> Option<Self>
pub fn with_layout(layout: Layout) -> Option<Self>
Create an element for a fictional type with specific layout.
It’s up to the caller to define or use an actual type with that same layout later. This skips the check that such a type must not contain any padding and only performs the layout related checks.
sourcepub fn layout(self) -> Layout
pub fn layout(self) -> Layout
Convert this into a type layout.
This can never fail as TexelLayout refines the standard library layout type.
sourcepub fn packed(self, align: usize) -> TexelLayout
pub fn packed(self, align: usize) -> TexelLayout
Reduce the alignment of the element.
This will perform the same modification as repr(packed) on the element’s type.
Panics
This method panics if align is not a valid alignment.
sourcepub fn infimum(self, other: Self) -> TexelLayout
pub fn infimum(self, other: Self) -> TexelLayout
Create an element having the smaller of both sizes and alignments.
pub const fn superset_of(&self, other: TexelLayout) -> bool