pub trait StridedLayout: Layout {
    fn strided(&self) -> StridedBytes;
}
A layout that is a strided matrix of elements.

Like all layout traits, implementations should ensure that the layout returned in these methods occupied a subset of pixels of their original layout.

Required Methods

The valid strided specification of this layout.

This call should not fail, or panic. Otherwise, prefer an optional getter for the StridedBytes and have the caller decay their own buffer.

Implementations on Foreign Types

Implementors