pub struct IsTransparentWrapper<P, O>(_);
Marker struct to denote that P is transparently wrapped in O.

The only way to construct it is by accessing its associated constant which only exists when the bound bytemuck::TransparentWrapper holds as required. This encodes a type-level set and is a workaround for such bounds not yet being allowed in const fn. Expect this type to be deprecated sooner or later.

Implementations