pub struct MaxAligned(/* private fields */);Expand description
A byte-like-type that is aligned to the required max alignment.
This type does not contain padding and implements Pod. Generally, the alignment and size
requirement is kept small to avoid overhead.
A byte-like-type that is aligned to the required max alignment.
This type does not contain padding and implements Pod. Generally, the alignment and size
requirement is kept small to avoid overhead.