Struct image_texel::texels::MaxAligned
source · [−]#[repr(align(/*«*/ 32 /*»*/))]#[repr(C)]pub struct MaxAligned(_);Expand description
A byte-like-type that is aligned to the required max alignment.
This type does not contain padding and implements Pod. Generally, the alignment and size
requirement is kept small to avoid overhead.