MaxCell

Struct MaxCell 

pub struct MaxCell(/* private fields */);
A cell of a byte array equivalent to MaxAligned.

Implementations

Source

impl MaxCell

pub const fn zero() -> Self

Create a vector of atomic zero-bytes.

pub fn new(contents: MaxAligned) -> Self

Create a vector from values initialized synchronously.

pub fn set(&self, newval: &Self)

Overwrite the contents with new information from another cell.

pub fn get(&self) -> MaxAligned

Read the current contents from this cell into an owned value.

pub fn into_inner(self) -> MaxAligned

Unwrap an owned value.